﻿namespace __ILC
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.Globalization;
    using System.Linq;
    using System.Linq.Expressions;
    using System.Reflection;
    using System.Reflection.Emit;
    using System.Runtime.CompilerServices;
    using System.Runtime.InteropServices;
    using System.Security.Permissions;
    using System.Text;
    using System.Threading;
    using System.Web;

    internal class ClassFactory
    {
        private int classCount;
        private Dictionary<Signature, Type> classes;
        public static readonly ClassFactory Instance = new ClassFactory();
        private ModuleBuilder module;
        private ReaderWriterLock rwLock;

        private ClassFactory()
        {
            AssemblyName name = new AssemblyName("DynamicClasses");
            this.module = AppDomain.CurrentDomain.DefineDynamicAssembly(name, AssemblyBuilderAccess.Run).DefineDynamicModule("Module");
            this.classes = new Dictionary<Signature, Type>();
            this.rwLock = new ReaderWriterLock();
        }

        private Type CreateDynamicClass(DynamicProperty[] properties)
        {
            Type type2;
            LockCookie lockCookie = this.rwLock.UpgradeToWriterLock(-1);
            try
            {
                string name = "DynamicClass" + (this.classCount + 1);
                TypeBuilder tb = this.module.DefineType(name, TypeAttributes.Public, typeof(DynamicClass));
                FieldInfo[] fields = this.GenerateProperties(tb, properties);
                this.GenerateEquals(tb, fields);
                this.GenerateGetHashCode(tb, fields);
                Type type = tb.CreateType();
                this.classCount++;
                type2 = type;
            }
            finally
            {
                this.rwLock.DowngradeFromWriterLock(ref lockCookie);
            }
            return type2;
        }

        private void GenerateEquals(TypeBuilder tb, FieldInfo[] fields)
        {
            ILGenerator iLGenerator = tb.DefineMethod("Equals", MethodAttributes.HideBySig | MethodAttributes.Virtual | MethodAttributes.Public, typeof(bool), new Type[] { typeof(object) }).GetILGenerator();
            LocalBuilder local = iLGenerator.DeclareLocal(tb);
            Label label = iLGenerator.DefineLabel();
            iLGenerator.Emit(OpCodes.Ldarg_1);
            iLGenerator.Emit(OpCodes.Isinst, tb);
            iLGenerator.Emit(OpCodes.Stloc, local);
            iLGenerator.Emit(OpCodes.Ldloc, local);
            iLGenerator.Emit(OpCodes.Brtrue_S, label);
            iLGenerator.Emit(OpCodes.Ldc_I4_0);
            iLGenerator.Emit(OpCodes.Ret);
            iLGenerator.MarkLabel(label);
            foreach (FieldInfo info in fields)
            {
                Type fieldType = info.FieldType;
                Type type2 = typeof(EqualityComparer<>).MakeGenericType(new Type[] { fieldType });
                label = iLGenerator.DefineLabel();
                iLGenerator.EmitCall(OpCodes.Call, type2.GetMethod("get_Default"), null);
                iLGenerator.Emit(OpCodes.Ldarg_0);
                iLGenerator.Emit(OpCodes.Ldfld, info);
                iLGenerator.Emit(OpCodes.Ldloc, local);
                iLGenerator.Emit(OpCodes.Ldfld, info);
                iLGenerator.EmitCall(OpCodes.Callvirt, type2.GetMethod("Equals", new Type[] { fieldType, fieldType }), null);
                iLGenerator.Emit(OpCodes.Brtrue_S, label);
                iLGenerator.Emit(OpCodes.Ldc_I4_0);
                iLGenerator.Emit(OpCodes.Ret);
                iLGenerator.MarkLabel(label);
            }
            iLGenerator.Emit(OpCodes.Ldc_I4_1);
            iLGenerator.Emit(OpCodes.Ret);
        }

        private void GenerateGetHashCode(TypeBuilder tb, FieldInfo[] fields)
        {
            ILGenerator iLGenerator = tb.DefineMethod("GetHashCode", MethodAttributes.HideBySig | MethodAttributes.Virtual | MethodAttributes.Public, typeof(int), Type.EmptyTypes).GetILGenerator();
            iLGenerator.Emit(OpCodes.Ldc_I4_0);
            foreach (FieldInfo info in fields)
            {
                Type fieldType = info.FieldType;
                Type type2 = typeof(EqualityComparer<>).MakeGenericType(new Type[] { fieldType });
                iLGenerator.EmitCall(OpCodes.Call, type2.GetMethod("get_Default"), null);
                iLGenerator.Emit(OpCodes.Ldarg_0);
                iLGenerator.Emit(OpCodes.Ldfld, info);
                iLGenerator.EmitCall(OpCodes.Callvirt, type2.GetMethod("GetHashCode", new Type[] { fieldType }), null);
                iLGenerator.Emit(OpCodes.Xor);
            }
            iLGenerator.Emit(OpCodes.Ret);
        }

        private FieldInfo[] GenerateProperties(TypeBuilder tb, DynamicProperty[] properties)
        {
            FieldInfo[] infoArray = new FieldBuilder[properties.Length];
            for (int i = 0; i < properties.Length; i++)
            {
                DynamicProperty property = properties[i];
                FieldBuilder field = tb.DefineField("_" + property.Name, property.Type, FieldAttributes.Private);
                PropertyBuilder builder2 = tb.DefineProperty(property.Name, PropertyAttributes.HasDefault, property.Type, null);
                MethodBuilder mdBuilder = tb.DefineMethod("get_" + property.Name, MethodAttributes.SpecialName | MethodAttributes.HideBySig | MethodAttributes.Public, property.Type, Type.EmptyTypes);
                ILGenerator iLGenerator = mdBuilder.GetILGenerator();
                iLGenerator.Emit(OpCodes.Ldarg_0);
                iLGenerator.Emit(OpCodes.Ldfld, field);
                iLGenerator.Emit(OpCodes.Ret);
                MethodBuilder builder4 = tb.DefineMethod("set_" + property.Name, MethodAttributes.SpecialName | MethodAttributes.HideBySig | MethodAttributes.Public, null, new Type[] { property.Type });
                ILGenerator generator2 = builder4.GetILGenerator();
                generator2.Emit(OpCodes.Ldarg_0);
                generator2.Emit(OpCodes.Ldarg_1);
                generator2.Emit(OpCodes.Stfld, field);
                generator2.Emit(OpCodes.Ret);
                builder2.SetGetMethod(mdBuilder);
                builder2.SetSetMethod(builder4);
                infoArray[i] = field;
            }
            return infoArray;
        }

        public Type GetDynamicClass(IEnumerable<DynamicProperty> properties)
        {
            Type type2;
            this.rwLock.AcquireReaderLock(-1);
            try
            {
                Type type;
                Signature key = new Signature(properties);
                if (!this.classes.TryGetValue(key, out type))
                {
                    type = this.CreateDynamicClass(key.properties);
                    this.classes.Add(key, type);
                }
                type2 = type;
            }
            finally
            {
                this.rwLock.ReleaseReaderLock();
            }
            return type2;
        }
    }

    [AspNetHostingPermission(SecurityAction.LinkDemand, Level = AspNetHostingPermissionLevel.Minimal), AspNetHostingPermission(SecurityAction.InheritanceDemand, Level = AspNetHostingPermissionLevel.Minimal)]
    public abstract class DynamicClass
    {
        protected DynamicClass()
        {
        }

        public override string ToString()
        {
            PropertyInfo[] properties = base.GetType().GetProperties(BindingFlags.Public | BindingFlags.Instance);
            StringBuilder builder = new StringBuilder();
            builder.Append("{");
            for (int i = 0; i < properties.Length; i++)
            {
                if (i > 0)
                {
                    builder.Append(", ");
                }
                builder.Append(properties[i].Name);
                builder.Append("=");
                builder.Append(properties[i].GetValue(this, null));
            }
            builder.Append("}");
            return builder.ToString();
        }
    }

    internal static class DynamicExpression
    {
        private static readonly Type[] funcTypes = new Type[] { typeof(Func<>), typeof(Func<,>), typeof(Func<,,>), typeof(Func<,,,>), typeof(Func<,,,,>) };

        public static Type CreateClass(params DynamicProperty[] properties)
        {
            return ClassFactory.Instance.GetDynamicClass(properties);
        }

        public static Type CreateClass(IEnumerable<DynamicProperty> properties)
        {
            return ClassFactory.Instance.GetDynamicClass(properties);
        }

        public static Type GetFuncType(params Type[] typeArgs)
        {
            if (((typeArgs == null) || (typeArgs.Length < 1)) || (typeArgs.Length > 5))
            {
                throw new ArgumentException();
            }
            return funcTypes[typeArgs.Length - 1].MakeGenericType(typeArgs);
        }

        public static LambdaExpression Lambda(Expression body, params ParameterExpression[] parameters)
        {
            int index = (parameters == null) ? 0 : parameters.Length;
            Type[] typeArgs = new Type[index + 1];
            for (int i = 0; i < index; i++)
            {
                typeArgs[i] = parameters[i].Type;
            }
            typeArgs[index] = body.Type;
            return Expression.Lambda(GetFuncType(typeArgs), body, parameters);
        }

        public static Expression Parse(Type resultType, string expression, params object[] values)
        {
            ExpressionParser parser = new ExpressionParser(null, expression, values);
            return parser.Parse(resultType);
        }

        public static Expression<Func<T, S>> ParseLambda<T, S>(string expression, params object[] values)
        {
            return (Expression<Func<T, S>>)ParseLambda(typeof(T), typeof(S), expression, values);
        }

        public static LambdaExpression ParseLambda(Type itType, Type resultType, string expression, params object[] values)
        {
            return ParseLambda(new ParameterExpression[] { Expression.Parameter(itType, "") }, resultType, expression, values);
        }

        public static LambdaExpression ParseLambda(ParameterExpression[] parameters, Type resultType, string expression, params object[] values)
        {
            ExpressionParser parser = new ExpressionParser(parameters, expression, values);
            return Lambda(parser.Parse(resultType), parameters);
        }
    }

    internal class DynamicOrdering
    {
        public bool Ascending;
        public Expression Selector;
    }

    internal class DynamicProperty
    {
        private string name;
        private Type type;

        public DynamicProperty(string name, Type type)
        {
            if (name == null)
            {
                throw new ArgumentNullException("name");
            }
            if (type == null)
            {
                throw new ArgumentNullException("type");
            }
            this.name = name;
            this.type = type;
        }

        public string Name
        {
            get
            {
                return this.name;
            }
        }

        public Type Type
        {
            get
            {
                return this.type;
            }
        }
    }

    internal static class DynamicQueryable
    {
        public static bool Any(this IQueryable source)
        {
            if (source == null)
            {
                throw new ArgumentNullException("source");
            }
            return (bool)source.Provider.Execute(Expression.Call(typeof(Queryable), "Any", new Type[] { source.ElementType }, new Expression[] { source.Expression }));
        }

        public static int Count(this IQueryable source)
        {
            if (source == null)
            {
                throw new ArgumentNullException("source");
            }
            return (int)source.Provider.Execute(Expression.Call(typeof(Queryable), "Count", new Type[] { source.ElementType }, new Expression[] { source.Expression }));
        }

        public static IQueryable GroupBy(this IQueryable source, string keySelector, string elementSelector, params object[] values)
        {
            if (source == null)
            {
                throw new ArgumentNullException("source");
            }
            if (keySelector == null)
            {
                throw new ArgumentNullException("keySelector");
            }
            if (elementSelector == null)
            {
                throw new ArgumentNullException("elementSelector");
            }
            LambdaExpression expression = DynamicExpression.ParseLambda(source.ElementType, null, keySelector, values);
            LambdaExpression expression2 = DynamicExpression.ParseLambda(source.ElementType, null, elementSelector, values);
            return source.Provider.CreateQuery(Expression.Call(typeof(Queryable), "GroupBy", new Type[] { source.ElementType, expression.Body.Type, expression2.Body.Type }, new Expression[] { source.Expression, Expression.Quote(expression), Expression.Quote(expression2) }));
        }

        public static IQueryable<T> OrderBy<T>(this IQueryable<T> source, string ordering, params object[] values)
        {
            return (IQueryable<T>)source.OrderBy(ordering, values);
        }

        public static IQueryable OrderBy(this IQueryable source, string ordering, params object[] values)
        {
            if (source == null)
            {
                throw new ArgumentNullException("source");
            }
            if (ordering == null)
            {
                throw new ArgumentNullException("ordering");
            }
            ParameterExpression[] parameters = new ParameterExpression[] { Expression.Parameter(source.ElementType, "") };
            IEnumerable<DynamicOrdering> enumerable = new ExpressionParser(parameters, ordering, values).ParseOrdering();
            Expression expression = source.Expression;
            string str = "OrderBy";
            string str2 = "OrderByDescending";
            foreach (DynamicOrdering ordering2 in enumerable)
            {
                expression = Expression.Call(typeof(Queryable), ordering2.Ascending ? str : str2, new Type[] { source.ElementType, ordering2.Selector.Type }, new Expression[] { expression, Expression.Quote(DynamicExpression.Lambda(ordering2.Selector, parameters)) });
                str = "ThenBy";
                str2 = "ThenByDescending";
            }
            return source.Provider.CreateQuery(expression);
        }

        public static IQueryable Select(this IQueryable source, string selector, params object[] values)
        {
            if (source == null)
            {
                throw new ArgumentNullException("source");
            }
            if (selector == null)
            {
                throw new ArgumentNullException("selector");
            }
            LambdaExpression expression = DynamicExpression.ParseLambda(source.ElementType, null, selector, values);
            return source.Provider.CreateQuery(Expression.Call(typeof(Queryable), "Select", new Type[] { source.ElementType, expression.Body.Type }, new Expression[] { source.Expression, Expression.Quote(expression) }));
        }

        public static IQueryable Skip(this IQueryable source, int count)
        {
            if (source == null)
            {
                throw new ArgumentNullException("source");
            }
            return source.Provider.CreateQuery(Expression.Call(typeof(Queryable), "Skip", new Type[] { source.ElementType }, new Expression[] { source.Expression, Expression.Constant(count) }));
        }

        public static IQueryable Take(this IQueryable source, int count)
        {
            if (source == null)
            {
                throw new ArgumentNullException("source");
            }
            return source.Provider.CreateQuery(Expression.Call(typeof(Queryable), "Take", new Type[] { source.ElementType }, new Expression[] { source.Expression, Expression.Constant(count) }));
        }

        public static IQueryable Where(this IQueryable source, string predicate, params object[] values)
        {
            if (source == null)
            {
                throw new ArgumentNullException("source");
            }
            if (predicate == null)
            {
                throw new ArgumentNullException("predicate");
            }
            LambdaExpression expression = DynamicExpression.ParseLambda(source.ElementType, typeof(bool), predicate, values);
            return source.Provider.CreateQuery(Expression.Call(typeof(Queryable), "Where", new Type[] { source.ElementType }, new Expression[] { source.Expression, Expression.Quote(expression) }));
        }

        public static IQueryable<T> Where<T>(this IQueryable<T> source, string predicate, params object[] values)
        {
            return (IQueryable<T>)source.Where(predicate, values);
        }
    }

    [AspNetHostingPermission(SecurityAction.InheritanceDemand, Level = AspNetHostingPermissionLevel.Minimal), AspNetHostingPermission(SecurityAction.LinkDemand, Level = AspNetHostingPermissionLevel.Minimal)]
    public class ParseException: Exception
    {
        private int position;

        public ParseException(string message, int position)
            : base(message)
        {
            this.position = position;
        }

        public override string ToString()
        {
            return string.Format(CultureInfo.InvariantCulture, "表达式解析错误:{0},在{1}", new object[] { this.Message, this.position });
        }

        public int Position
        {
            get
            {
                return this.position;
            }
        }
    }

    internal class Signature: IEquatable<Signature>
    {
        public int hashCode;
        public DynamicProperty[] properties;

        public Signature(IEnumerable<DynamicProperty> properties)
        {
            this.properties = properties.ToArray<DynamicProperty>();
            this.hashCode = 0;
            foreach (DynamicProperty property in properties)
            {
                this.hashCode ^= property.Name.GetHashCode() ^ property.Type.GetHashCode();
            }
        }

        public override bool Equals(object obj)
        {
            return ((obj is Signature) && this.Equals((Signature)obj));
        }

        public bool Equals(Signature other)
        {
            if (this.properties.Length != other.properties.Length)
            {
                return false;
            }
            for (int i = 0; i < this.properties.Length; i++)
            {
                if ((this.properties[i].Name != other.properties[i].Name) || (this.properties[i].Type != other.properties[i].Type))
                {
                    return false;
                }
            }
            return true;
        }

        public override int GetHashCode()
        {
            return this.hashCode;
        }
    }

    internal class ExpressionParser
    {
        private char ch;
        private IDictionary<string, object> externals;
        private static readonly Expression falseLiteral = Expression.Constant(false);
        private ParameterExpression it;
        private static readonly string keywordIif = "iif";
        private static readonly string keywordIt = "it";
        private static readonly string keywordNew = "new";
        private static Dictionary<string, object> keywords;
        private Dictionary<Expression, string> literals;
        private static readonly Expression nullLiteral = Expression.Constant(null);

        private static readonly Type[] predefinedTypes = new Type[] {
    typeof(object), typeof(bool), typeof(char), typeof(string), typeof(sbyte), typeof(byte), typeof(short), typeof(ushort), typeof(int), typeof(uint), typeof(long), typeof(ulong), typeof(float), typeof(double), typeof(decimal), typeof(DateTime),
    typeof(DateTimeOffset), typeof(TimeSpan), typeof(Guid), typeof(Math), typeof(Convert)
   };

        private Dictionary<string, object> symbols;
        private string text;
        private int textLen;
        private int textPos;
        private Token token;
        private static readonly Expression trueLiteral = Expression.Constant(true);

        public ExpressionParser(ParameterExpression[] parameters, string expression, object[] values)
        {
            if (expression == null)
            {
                throw new ArgumentNullException("expression");
            }
            if (keywords == null)
            {
                keywords = CreateKeywords();
            }
            this.symbols = new Dictionary<string, object>(StringComparer.OrdinalIgnoreCase);
            this.literals = new Dictionary<Expression, string>();
            if (parameters != null)
            {
                this.ProcessParameters(parameters);
            }
            if (values != null)
            {
                this.ProcessValues(values);
            }
            this.text = expression;
            this.textLen = this.text.Length;
            this.SetTextPos(0);
            this.NextToken();
        }

        private static void AddInterface(List<Type> types, Type type)
        {
            if (!types.Contains(type))
            {
                types.Add(type);
                foreach (Type type2 in type.GetInterfaces())
                {
                    AddInterface(types, type2);
                }
            }
        }

        private void AddSymbol(string name, object value)
        {
            if (this.symbols.ContainsKey(name))
            {
                throw this.ParseError("名称重复", new object[] { name });
            }
            this.symbols.Add(name, value);
        }

        private void CheckAndPromoteOperand(Type signatures, string opName, ref Expression expr, int errorPos)
        {
            MethodBase base2;
            Expression[] args = new Expression[] { expr };
            if (this.FindMethod(signatures, "F", false, args, out base2) != 1)
            {
                throw this.ParseError(errorPos, "不兼容{0},{1}", new object[] { opName, GetTypeName(args[0].Type) });
            }
            expr = args[0];
        }

        private void CheckAndPromoteOperands(Type signatures, string opName, ref Expression left, ref Expression right, int errorPos)
        {
            MethodBase base2;
            Expression[] args = new Expression[] { left, right };
            if (this.FindMethod(signatures, "F", false, args, out base2) != 1)
            {
                throw this.IncompatibleOperandsError(opName, left, right, errorPos);
            }
            left = args[0];
            right = args[1];
        }

        private static int CompareConversions(Type s, Type t1, Type t2)
        {
            if (t1 != t2)
            {
                if (s == t1)
                {
                    return 1;
                }
                if (s == t2)
                {
                    return -1;
                }
                bool flag = IsCompatibleWith(t1, t2);
                bool flag2 = IsCompatibleWith(t2, t1);
                if (flag && !flag2)
                {
                    return 1;
                }
                if (flag2 && !flag)
                {
                    return -1;
                }
                if (IsSignedIntegralType(t1) && IsUnsignedIntegralType(t2))
                {
                    return 1;
                }
                if (IsSignedIntegralType(t2) && IsUnsignedIntegralType(t1))
                {
                    return -1;
                }
            }
            return 0;
        }

        private static Dictionary<string, object> CreateKeywords()
        {
            Dictionary<string, object> dictionary = new Dictionary<string, object>(StringComparer.OrdinalIgnoreCase);
            dictionary.Add("true", trueLiteral);
            dictionary.Add("false", falseLiteral);
            dictionary.Add("null", nullLiteral);
            dictionary.Add(keywordIt, keywordIt);
            dictionary.Add(keywordIif, keywordIif);
            dictionary.Add(keywordNew, keywordNew);
            foreach (Type type in predefinedTypes)
            {
                dictionary.Add(type.Name, type);
            }
            return dictionary;
        }

        private Expression CreateLiteral(object value, string text)
        {
            ConstantExpression key = Expression.Constant(value);
            this.literals.Add(key, text);
            return key;
        }

        private int FindBestMethod(IEnumerable<MethodBase> methods, Expression[] args, out MethodBase method)
        {
            Func<MethodData, bool> predicate = null;
            MethodData[] applicable = (from m in methods
                                       select new MethodData { MethodBase = m, Parameters = m.GetParameters() } into m
                                       where this.IsApplicable(m, args)
                                       select m).ToArray<MethodData>();
            if (applicable.Length > 1)
            {
                if (predicate == null)
                {
                    predicate = m => applicable.All<MethodData>(delegate(MethodData n)
                    {
                        if (m != n)
                        {
                            return IsBetterThan(args, m, n);
                        }
                        return true;
                    });
                }
                applicable = applicable.Where<MethodData>(predicate).ToArray<MethodData>();
            }
            if (applicable.Length == 1)
            {
                MethodData data = applicable[0];
                for (int i = 0; i < args.Length; i++)
                {
                    args[i] = data.Args[i];
                }
                method = data.MethodBase;
            }
            else
            {
                method = null;
            }
            return applicable.Length;
        }

        private static Type FindGenericType(Type generic, Type type)
        {
            while ((type != null) && (type != typeof(object)))
            {
                if (type.IsGenericType && (type.GetGenericTypeDefinition() == generic))
                {
                    return type;
                }
                if (generic.IsInterface)
                {
                    foreach (Type type2 in type.GetInterfaces())
                    {
                        Type type3 = FindGenericType(generic, type2);
                        if (type3 != null)
                        {
                            return type3;
                        }
                    }
                }
                type = type.BaseType;
            }
            return null;
        }

        private int FindIndexer(Type type, Expression[] args, out MethodBase method)
        {
            foreach (Type type2 in SelfAndBaseTypes(type))
            {
                MemberInfo[] defaultMembers = type2.GetDefaultMembers();
                if (defaultMembers.Length != 0)
                {
                    IEnumerable<MethodBase> methods = from p in defaultMembers.OfType<PropertyInfo>()
                                                      select p.GetGetMethod() into m
                                                      where m != null
                                                      select m;
                    int num = this.FindBestMethod(methods, args, out method);
                    if (num != 0)
                    {
                        return num;
                    }
                }
            }
            method = null;
            return 0;
        }

        private int FindMethod(Type type, string methodName, bool staticAccess, Expression[] args, out MethodBase method)
        {
            BindingFlags bindingAttr = (BindingFlags.Public | BindingFlags.DeclaredOnly) | (staticAccess ? BindingFlags.Static : BindingFlags.Instance);
            foreach (Type type2 in SelfAndBaseTypes(type))
            {
                MemberInfo[] source = type2.FindMembers(MemberTypes.Method, bindingAttr, Type.FilterNameIgnoreCase, methodName);
                int num = this.FindBestMethod(source.Cast<MethodBase>(), args, out method);
                if (num != 0)
                {
                    return num;
                }
            }
            method = null;
            return 0;
        }

        private MemberInfo FindPropertyOrField(Type type, string memberName, bool staticAccess)
        {
            BindingFlags bindingAttr = (BindingFlags.Public | BindingFlags.DeclaredOnly) | (staticAccess ? BindingFlags.Static : BindingFlags.Instance);
            foreach (Type type2 in SelfAndBaseTypes(type))
            {
                MemberInfo[] infoArray = type2.FindMembers(MemberTypes.Property | MemberTypes.Field, bindingAttr, Type.FilterNameIgnoreCase, memberName);
                if (infoArray.Length != 0)
                {
                    return infoArray[0];
                }
            }
            return null;
        }

        private Expression GenerateAdd(Expression left, Expression right)
        {
            if ((left.Type == typeof(string)) && (right.Type == typeof(string)))
            {
                return this.GenerateStaticMethodCall("Concat", left, right);
            }
            return Expression.Add(left, right);
        }

        private Expression GenerateConditional(Expression test, Expression expr1, Expression expr2, int errorPos)
        {
            if (test.Type != typeof(bool))
            {
                throw this.ParseError(errorPos, "必须是布尔表达式");
            }
            if (expr1.Type != expr2.Type)
            {
                Expression expression = (expr2 != nullLiteral) ? this.PromoteExpression(expr1, expr2.Type, true) : null;
                Expression expression2 = (expr1 != nullLiteral) ? this.PromoteExpression(expr2, expr1.Type, true) : null;
                if ((expression == null) || (expression2 != null))
                {
                    if ((expression2 == null) || (expression != null))
                    {
                        string str = (expr1 != nullLiteral) ? expr1.Type.Name : "null";
                        string str2 = (expr2 != nullLiteral) ? expr2.Type.Name : "null";
                        if ((expression != null) && (expression2 != null))
                        {
                            throw this.ParseError(errorPos, "类型转换:{0},{1}", new object[] { str, str2 });
                        }
                        throw this.ParseError(errorPos, "类型转换:{0},{1}", new object[] { str, str2 });
                    }
                    expr2 = expression2;
                }
                else
                {
                    expr1 = expression;
                }
            }
            return Expression.Condition(test, expr1, expr2);
        }

        private Expression GenerateConversion(Expression expr, Type type, int errorPos)
        {
            Type type2 = expr.Type;
            if (type2 == type)
            {
                return expr;
            }
            if (type2.IsValueType && type.IsValueType)
            {
                if ((IsNullableType(type2) || IsNullableType(type)) && (GetNonNullableType(type2) == GetNonNullableType(type)))
                {
                    return Expression.Convert(expr, type);
                }
                if ((IsNumericType(type2) || IsEnumType(type2)) && (IsNumericType(type) || IsEnumType(type)))
                {
                    return Expression.ConvertChecked(expr, type);
                }
            }
            if ((!type2.IsAssignableFrom(type) && !type.IsAssignableFrom(type2)) && (!type2.IsInterface && !type.IsInterface))
            {
                throw this.ParseError(errorPos, "类型转换:{0},{1}", new object[] { GetTypeName(type2), GetTypeName(type) });
            }
            return Expression.Convert(expr, type);
        }

        private Expression GenerateEqual(Expression left, Expression right)
        {
            return Expression.Equal(left, right);
        }

        private Expression GenerateGreaterThan(Expression left, Expression right)
        {
            if (left.Type == typeof(string))
            {
                return Expression.GreaterThan(this.GenerateStaticMethodCall("Compare", left, right), Expression.Constant(0));
            }
            return Expression.GreaterThan(left, right);
        }

        private Expression GenerateGreaterThanEqual(Expression left, Expression right)
        {
            if (left.Type == typeof(string))
            {
                return Expression.GreaterThanOrEqual(this.GenerateStaticMethodCall("Compare", left, right), Expression.Constant(0));
            }
            return Expression.GreaterThanOrEqual(left, right);
        }

        private Expression GenerateLessThan(Expression left, Expression right)
        {
            if (left.Type == typeof(string))
            {
                return Expression.LessThan(this.GenerateStaticMethodCall("Compare", left, right), Expression.Constant(0));
            }
            return Expression.LessThan(left, right);
        }

        private Expression GenerateLessThanEqual(Expression left, Expression right)
        {
            if (left.Type == typeof(string))
            {
                return Expression.LessThanOrEqual(this.GenerateStaticMethodCall("Compare", left, right), Expression.Constant(0));
            }
            return Expression.LessThanOrEqual(left, right);
        }

        private Expression GenerateNotEqual(Expression left, Expression right)
        {
            return Expression.NotEqual(left, right);
        }

        private Expression GenerateStaticMethodCall(string methodName, Expression left, Expression right)
        {
            return Expression.Call(null, this.GetStaticMethod(methodName, left, right), new Expression[] { left, right });
        }

        private Expression GenerateStringConcat(Expression left, Expression right)
        {
            if (left.Type.IsValueType)
            {
                left = Expression.Convert(left, typeof(object));
            }
            if (right.Type.IsValueType)
            {
                right = Expression.Convert(right, typeof(object));
            }
            return Expression.Call(null, typeof(string).GetMethod("Concat", new Type[] { typeof(object), typeof(object) }), new Expression[] { left, right });
        }

        private Expression GenerateSubtract(Expression left, Expression right)
        {
            return Expression.Subtract(left, right);
        }

        private string GetIdentifier()
        {
            this.ValidateToken(TokenId.Identifier, "需要标识符");
            string text = this.token.text;
            if ((text.Length > 1) && (text[0] == '@'))
            {
                text = text.Substring(1);
            }
            return text;
        }

        private static Type GetNonNullableType(Type type)
        {
            if (!IsNullableType(type))
            {
                return type;
            }
            return type.GetGenericArguments()[0];
        }

        private static int GetNumericTypeKind(Type type)
        {
            type = GetNonNullableType(type);
            if (!type.IsEnum)
            {
                switch (Type.GetTypeCode(type))
                {
                    case TypeCode.Char:
                    case TypeCode.Single:
                    case TypeCode.Double:
                    case TypeCode.Decimal:
                        return 1;

                    case TypeCode.SByte:
                    case TypeCode.Int16:
                    case TypeCode.Int32:
                    case TypeCode.Int64:
                        return 2;

                    case TypeCode.Byte:
                    case TypeCode.UInt16:
                    case TypeCode.UInt32:
                    case TypeCode.UInt64:
                        return 3;
                }
            }
            return 0;
        }

        private MethodInfo GetStaticMethod(string methodName, Expression left, Expression right)
        {
            return left.Type.GetMethod(methodName, new Type[] { left.Type, right.Type });
        }

        private static string GetTypeName(Type type)
        {
            Type nonNullableType = GetNonNullableType(type);
            string name = nonNullableType.Name;
            if (type != nonNullableType)
            {
                name = name + '?';
            }
            return name;
        }

        private Exception IncompatibleOperandsError(string opName, Expression left, Expression right, int pos)
        {
            return this.ParseError(pos, "不兼容{0},{1},{2}", new object[] { opName, GetTypeName(left.Type), GetTypeName(right.Type) });
        }

        private bool IsApplicable(MethodData method, Expression[] args)
        {
            if (method.Parameters.Length != args.Length)
            {
                return false;
            }
            Expression[] expressionArray = new Expression[args.Length];
            for (int i = 0; i < args.Length; i++)
            {
                ParameterInfo info = method.Parameters[i];
                if (info.IsOut)
                {
                    return false;
                }
                Expression expression = this.PromoteExpression(args[i], info.ParameterType, false);
                if (expression == null)
                {
                    return false;
                }
                expressionArray[i] = expression;
            }
            method.Args = expressionArray;
            return true;
        }

        private static bool IsBetterThan(Expression[] args, MethodData m1, MethodData m2)
        {
            bool flag = false;
            for (int i = 0; i < args.Length; i++)
            {
                int num2 = CompareConversions(args[i].Type, m1.Parameters[i].ParameterType, m2.Parameters[i].ParameterType);
                if (num2 < 0)
                {
                    return false;
                }
                if (num2 > 0)
                {
                    flag = true;
                }
            }
            return flag;
        }

        private static bool IsCompatibleWith(Type source, Type target)
        {
            if (source == target)
            {
                return true;
            }
            if (!target.IsValueType)
            {
                return target.IsAssignableFrom(source);
            }
            Type nonNullableType = GetNonNullableType(source);
            Type type = GetNonNullableType(target);
            if ((nonNullableType == source) || (type != target))
            {
                TypeCode code = nonNullableType.IsEnum ? TypeCode.Object : Type.GetTypeCode(nonNullableType);
                TypeCode code2 = type.IsEnum ? TypeCode.Object : Type.GetTypeCode(type);
                switch (code)
                {
                    case TypeCode.SByte:
                        switch (code2)
                        {
                            case TypeCode.SByte:
                            case TypeCode.Int16:
                            case TypeCode.Int32:
                            case TypeCode.Int64:
                            case TypeCode.Single:
                            case TypeCode.Double:
                            case TypeCode.Decimal:
                                return true;
                        }
                        break;

                    case TypeCode.Byte:
                        switch (code2)
                        {
                            case TypeCode.Byte:
                            case TypeCode.Int16:
                            case TypeCode.UInt16:
                            case TypeCode.Int32:
                            case TypeCode.UInt32:
                            case TypeCode.Int64:
                            case TypeCode.UInt64:
                            case TypeCode.Single:
                            case TypeCode.Double:
                            case TypeCode.Decimal:
                                return true;
                        }
                        break;

                    case TypeCode.Int16:
                        switch (code2)
                        {
                            case TypeCode.Int16:
                            case TypeCode.Int32:
                            case TypeCode.Int64:
                            case TypeCode.Single:
                            case TypeCode.Double:
                            case TypeCode.Decimal:
                                return true;
                        }
                        break;

                    case TypeCode.UInt16:
                        switch (code2)
                        {
                            case TypeCode.UInt16:
                            case TypeCode.Int32:
                            case TypeCode.UInt32:
                            case TypeCode.Int64:
                            case TypeCode.UInt64:
                            case TypeCode.Single:
                            case TypeCode.Double:
                            case TypeCode.Decimal:
                                return true;
                        }
                        break;

                    case TypeCode.Int32:
                        switch (code2)
                        {
                            case TypeCode.Int32:
                            case TypeCode.Int64:
                            case TypeCode.Single:
                            case TypeCode.Double:
                            case TypeCode.Decimal:
                                return true;
                        }
                        break;

                    case TypeCode.UInt32:
                        switch (code2)
                        {
                            case TypeCode.UInt32:
                            case TypeCode.Int64:
                            case TypeCode.UInt64:
                            case TypeCode.Single:
                            case TypeCode.Double:
                            case TypeCode.Decimal:
                                return true;
                        }
                        break;

                    case TypeCode.Int64:
                        switch (code2)
                        {
                            case TypeCode.Int64:
                            case TypeCode.Single:
                            case TypeCode.Double:
                            case TypeCode.Decimal:
                                return true;
                        }
                        break;

                    case TypeCode.UInt64:
                        switch (code2)
                        {
                            case TypeCode.UInt64:
                            case TypeCode.Single:
                            case TypeCode.Double:
                            case TypeCode.Decimal:
                                return true;
                        }
                        break;

                    case TypeCode.Single:
                        switch (code2)
                        {
                            case TypeCode.Single:
                            case TypeCode.Double:
                                return true;
                        }
                        break;

                    default:
                        if (nonNullableType == type)
                        {
                            return true;
                        }
                        break;
                }
            }
            return false;
        }

        private static bool IsEnumType(Type type)
        {
            return GetNonNullableType(type).IsEnum;
        }

        private static bool IsIdentifierPart(char ch)
        {
            return (((((int)1) << (int)char.GetUnicodeCategory(ch)) & 295807) != 0);
        }

        private static bool IsIdentifierStart(char ch)
        {
            return (((((int)1) << (int)char.GetUnicodeCategory(ch)) & 543) != 0);
        }

        private static bool IsNullableType(Type type)
        {
            return (type.IsGenericType && (type.GetGenericTypeDefinition() == typeof(Nullable<>)));
        }

        private static bool IsNumericType(Type type)
        {
            return (GetNumericTypeKind(type) != 0);
        }

        private static bool IsPredefinedType(Type type)
        {
            foreach (Type type2 in predefinedTypes)
            {
                if (type2 == type)
                {
                    return true;
                }
            }
            return false;
        }

        private static bool IsSignedIntegralType(Type type)
        {
            return (GetNumericTypeKind(type) == 2);
        }

        private static bool IsUnsignedIntegralType(Type type)
        {
            return (GetNumericTypeKind(type) == 3);
        }

        private void NextChar()
        {
            if (this.textPos < this.textLen)
            {
                this.textPos++;
            }
            this.ch = (this.textPos < this.textLen) ? this.text[this.textPos] : '\0';
        }

        private void NextToken()
        {
            TokenId exclamation;
            while (char.IsWhiteSpace(this.ch))
            {
                this.NextChar();
            }
            int textPos = this.textPos;
            switch (this.ch)
            {
                case '!':
                    this.NextChar();
                    if (this.ch != '=')
                    {
                        exclamation = TokenId.Exclamation;
                    }
                    else
                    {
                        this.NextChar();
                        exclamation = TokenId.ExclamationEqual;
                    }
                    break;

                case '"':
                case '\'':
                    {
                        char ch = this.ch;
                        do
                        {
                            this.NextChar();
                            while ((this.textPos < this.textLen) && (this.ch != ch))
                            {
                                this.NextChar();
                            }
                            if (this.textPos == this.textLen)
                            {
                                throw this.ParseError(this.textPos, "未结束", new object[0]);
                            }
                            this.NextChar();
                        }
                        while (this.ch == ch);
                        exclamation = TokenId.StringLiteral;
                        break;
                    }
                case '%':
                    this.NextChar();
                    exclamation = TokenId.Percent;
                    break;

                case '&':
                    this.NextChar();
                    if (this.ch != '&')
                    {
                        exclamation = TokenId.Amphersand;
                    }
                    else
                    {
                        this.NextChar();
                        exclamation = TokenId.DoubleAmphersand;
                    }
                    break;

                case '(':
                    this.NextChar();
                    exclamation = TokenId.OpenParen;
                    break;

                case ')':
                    this.NextChar();
                    exclamation = TokenId.CloseParen;
                    break;

                case '*':
                    this.NextChar();
                    exclamation = TokenId.Asterisk;
                    break;

                case '+':
                    this.NextChar();
                    exclamation = TokenId.Plus;
                    break;

                case ',':
                    this.NextChar();
                    exclamation = TokenId.Comma;
                    break;

                case '-':
                    this.NextChar();
                    exclamation = TokenId.Minus;
                    break;

                case '.':
                    this.NextChar();
                    exclamation = TokenId.Dot;
                    break;

                case '/':
                    this.NextChar();
                    exclamation = TokenId.Slash;
                    break;

                case ':':
                    this.NextChar();
                    exclamation = TokenId.Colon;
                    break;

                case '<':
                    this.NextChar();
                    if (this.ch != '=')
                    {
                        if (this.ch == '>')
                        {
                            this.NextChar();
                            exclamation = TokenId.LessGreater;
                        }
                        else
                        {
                            exclamation = TokenId.LessThan;
                        }
                    }
                    else
                    {
                        this.NextChar();
                        exclamation = TokenId.LessThanEqual;
                    }
                    break;

                case '=':
                    this.NextChar();
                    if (this.ch != '=')
                    {
                        exclamation = TokenId.Equal;
                    }
                    else
                    {
                        this.NextChar();
                        exclamation = TokenId.DoubleEqual;
                    }
                    break;

                case '>':
                    this.NextChar();
                    if (this.ch != '=')
                    {
                        exclamation = TokenId.GreaterThan;
                    }
                    else
                    {
                        this.NextChar();
                        exclamation = TokenId.GreaterThanEqual;
                    }
                    break;

                case '?':
                    this.NextChar();
                    exclamation = TokenId.Question;
                    break;

                case '[':
                    this.NextChar();
                    exclamation = TokenId.OpenBracket;
                    break;

                case ']':
                    this.NextChar();
                    exclamation = TokenId.CloseBracket;
                    break;

                case '|':
                    this.NextChar();
                    if (this.ch == '|')
                    {
                        this.NextChar();
                        exclamation = TokenId.DoubleBar;
                    }
                    else
                    {
                        exclamation = TokenId.Bar;
                    }
                    break;

                default:
                    if ((IsIdentifierStart(this.ch) || (this.ch == '@')) || (this.ch == '_'))
                    {
                        do
                        {
                            this.NextChar();
                        }
                        while (IsIdentifierPart(this.ch) || (this.ch == '_'));
                        exclamation = TokenId.Identifier;
                    }
                    else if (char.IsDigit(this.ch))
                    {
                        exclamation = TokenId.IntegerLiteral;
                        do
                        {
                            this.NextChar();
                        }
                        while (char.IsDigit(this.ch));
                        if (this.ch == '.')
                        {
                            exclamation = TokenId.RealLiteral;
                            this.NextChar();
                            this.ValidateDigit();
                            do
                            {
                                this.NextChar();
                            }
                            while (char.IsDigit(this.ch));
                        }
                        if ((this.ch == 'E') || (this.ch == 'e'))
                        {
                            exclamation = TokenId.RealLiteral;
                            this.NextChar();
                            if ((this.ch == '+') || (this.ch == '-'))
                            {
                                this.NextChar();
                            }
                            this.ValidateDigit();
                            do
                            {
                                this.NextChar();
                            }
                            while (char.IsDigit(this.ch));
                        }
                        if ((this.ch == 'F') || (this.ch == 'f'))
                        {
                            this.NextChar();
                        }
                    }
                    else
                    {
                        if (this.textPos != this.textLen)
                        {
                            throw this.ParseError(this.textPos, "非法字符:{0}", new object[] { this.ch });
                        }
                        exclamation = TokenId.End;
                    }
                    break;
            }
            this.token.id = exclamation;
            this.token.text = this.text.Substring(textPos, this.textPos - textPos);
            this.token.pos = textPos;
        }

        public Expression Parse(Type resultType)
        {
            int pos = this.token.pos;
            Expression expr = this.ParseExpression();
            if ((resultType != null) && ((expr = this.PromoteExpression(expr, resultType, true)) == null))
            {
                throw this.ParseError(pos, "表达式类型不匹配:{0}", new object[] { GetTypeName(resultType) });
            }
            this.ValidateToken(TokenId.End, "语法错误");
            return expr;
        }

        private Expression ParseAdditive()
        {
            Expression left = this.ParseMultiplicative();
            while (((this.token.id == TokenId.Plus) || (this.token.id == TokenId.Minus)) || (this.token.id == TokenId.Amphersand))
            {
                Token token = this.token;
                this.NextToken();
                Expression right = this.ParseMultiplicative();
                switch (token.id)
                {
                    case TokenId.Plus:
                        {
                            if ((left.Type == typeof(string)) || (right.Type == typeof(string)))
                            {
                                break;
                            }
                            this.CheckAndPromoteOperands(typeof(IAddSignatures), token.text, ref left, ref right, token.pos);
                            left = this.GenerateAdd(left, right);
                            continue;
                        }
                    case TokenId.Comma:
                        {
                            continue;
                        }
                    case TokenId.Minus:
                        {
                            this.CheckAndPromoteOperands(typeof(ISubtractSignatures), token.text, ref left, ref right, token.pos);
                            left = this.GenerateSubtract(left, right);
                            continue;
                        }
                    case TokenId.Amphersand:
                        break;

                    default:
                        {
                            continue;
                        }
                }
                left = this.GenerateStringConcat(left, right);
            }
            return left;
        }

        private Expression ParseAggregate(Expression instance, Type elementType, string methodName, int errorPos)
        {
            MethodBase base2;
            Type[] typeArray;
            ParameterExpression it = this.it;
            ParameterExpression expression2 = Expression.Parameter(elementType, "");
            this.it = expression2;
            Expression[] args = this.ParseArgumentList();
            this.it = it;
            if (this.FindMethod(typeof(IEnumerableSignatures), methodName, false, args, out base2) != 1)
            {
                throw this.ParseError(errorPos, "没有适用的聚合:{0}", new object[] { methodName });
            }
            if ((base2.Name == "Min") || (base2.Name == "Max"))
            {
                typeArray = new Type[] { elementType, args[0].Type };
            }
            else
            {
                typeArray = new Type[] { elementType };
            }
            if (args.Length == 0)
            {
                args = new Expression[] { instance };
            }
            else
            {
                args = new Expression[] { instance, DynamicExpression.Lambda(args[0], new ParameterExpression[] { expression2 }) };
            }
            return Expression.Call(typeof(Enumerable), base2.Name, typeArray, args);
        }

        private Expression[] ParseArgumentList()
        {
            this.ValidateToken(TokenId.OpenParen, "需要开括号");
            this.NextToken();
            Expression[] expressionArray = (this.token.id != TokenId.CloseParen) ? this.ParseArguments() : new Expression[0];
            this.ValidateToken(TokenId.CloseParen, "需要关括号或逗号");
            this.NextToken();
            return expressionArray;
        }

        private Expression[] ParseArguments()
        {
            List<Expression> list = new List<Expression>();
            while (true)
            {
                list.Add(this.ParseExpression());
                if (this.token.id != TokenId.Comma)
                {
                    break;
                }
                this.NextToken();
            }
            return list.ToArray();
        }

        private Expression ParseComparison()
        {
            Expression left = this.ParseAdditive();
            while ((((this.token.id == TokenId.Equal) || (this.token.id == TokenId.DoubleEqual)) || ((this.token.id == TokenId.ExclamationEqual) || (this.token.id == TokenId.LessGreater))) || (((this.token.id == TokenId.GreaterThan) || (this.token.id == TokenId.GreaterThanEqual)) || ((this.token.id == TokenId.LessThan) || (this.token.id == TokenId.LessThanEqual))))
            {
                Token token = this.token;
                this.NextToken();
                Expression right = this.ParseAdditive();
                bool flag = (((token.id == TokenId.Equal) || (token.id == TokenId.DoubleEqual)) || (token.id == TokenId.ExclamationEqual)) || (token.id == TokenId.LessGreater);
                if ((flag && !left.Type.IsValueType) && !right.Type.IsValueType)
                {
                    if (left.Type != right.Type)
                    {
                        if (!left.Type.IsAssignableFrom(right.Type))
                        {
                            if (!right.Type.IsAssignableFrom(left.Type))
                            {
                                throw this.IncompatibleOperandsError(token.text, left, right, token.pos);
                            }
                            left = Expression.Convert(left, right.Type);
                        }
                        else
                        {
                            right = Expression.Convert(right, left.Type);
                        }
                    }
                }
                else if (IsEnumType(left.Type) || IsEnumType(right.Type))
                {
                    if (left.Type != right.Type)
                    {
                        Expression expression3 = this.PromoteExpression(right, left.Type, true);
                        if (expression3 == null)
                        {
                            expression3 = this.PromoteExpression(left, right.Type, true);
                            if (expression3 == null)
                            {
                                throw this.IncompatibleOperandsError(token.text, left, right, token.pos);
                            }
                            left = expression3;
                        }
                        else
                        {
                            right = expression3;
                        }
                    }
                }
                else
                {
                    this.CheckAndPromoteOperands(flag ? typeof(IEqualitySignatures) : typeof(IRelationalSignatures), token.text, ref left, ref right, token.pos);
                }
                switch (token.id)
                {
                    case TokenId.LessThan:
                        left = this.GenerateLessThan(left, right);
                        break;

                    case TokenId.Equal:
                    case TokenId.DoubleEqual:
                        left = this.GenerateEqual(left, right);
                        break;

                    case TokenId.GreaterThan:
                        left = this.GenerateGreaterThan(left, right);
                        break;

                    case TokenId.ExclamationEqual:
                    case TokenId.LessGreater:
                        left = this.GenerateNotEqual(left, right);
                        break;

                    case TokenId.LessThanEqual:
                        left = this.GenerateLessThanEqual(left, right);
                        break;

                    case TokenId.GreaterThanEqual:
                        left = this.GenerateGreaterThanEqual(left, right);
                        break;
                }
            }
            return left;
        }

        private Expression ParseElementAccess(Expression expr)
        {
            MethodBase base2;
            int pos = this.token.pos;
            this.ValidateToken(TokenId.OpenBracket, "需要开括号");
            this.NextToken();
            Expression[] args = this.ParseArguments();
            this.ValidateToken(TokenId.CloseBracket, "需要关括号或逗号");
            this.NextToken();
            if (expr.Type.IsArray)
            {
                if ((expr.Type.GetArrayRank() != 1) || (args.Length != 1))
                {
                    throw this.ParseError(pos, "不能索引的多维数组", new object[0]);
                }
                Expression index = this.PromoteExpression(args[0], typeof(int), true);
                if (index == null)
                {
                    throw this.ParseError(pos, "非法索引", new object[0]);
                }
                return Expression.ArrayIndex(expr, index);
            }
            switch (this.FindIndexer(expr.Type, args, out base2))
            {
                case 0:
                    throw this.ParseError(pos, "不适用索引 {0}", new object[] { GetTypeName(expr.Type) });

                case 1:
                    return Expression.Call(expr, (MethodInfo)base2, args);
            }
            throw this.ParseError(pos, "索引器调用不明确 {0}", new object[] { GetTypeName(expr.Type) });
        }

        private static object ParseEnum(string name, Type type)
        {
            if (type.IsEnum)
            {
                MemberInfo[] infoArray = type.FindMembers(MemberTypes.Field, BindingFlags.Public | BindingFlags.Static | BindingFlags.DeclaredOnly, Type.FilterNameIgnoreCase, name);
                if (infoArray.Length != 0)
                {
                    return ((FieldInfo)infoArray[0]).GetValue(null);
                }
            }
            return null;
        }

        private Exception ParseError(string format, params object[] args)
        {
            return this.ParseError(this.token.pos, format, args);
        }

        private Exception ParseError(int pos, string format, params object[] args)
        {
            return new ParseException(string.Format(CultureInfo.CurrentCulture, format, args), pos);
        }

        private Expression ParseExpression()
        {
            int pos = this.token.pos;
            Expression test = this.ParseLogicalOr();
            if (this.token.id == TokenId.Question)
            {
                this.NextToken();
                Expression expression2 = this.ParseExpression();
                this.ValidateToken(TokenId.Colon, "ColonExpected");
                this.NextToken();
                Expression expression3 = this.ParseExpression();
                test = this.GenerateConditional(test, expression2, expression3, pos);
            }
            return test;
        }

        private Expression ParseIdentifier()
        {
            object obj2;
            this.ValidateToken(TokenId.Identifier);
            if (keywords.TryGetValue(this.token.text, out obj2))
            {
                if (obj2 is Type)
                {
                    return this.ParseTypeAccess((Type)obj2);
                }
                if (obj2 == keywordIt)
                {
                    return this.ParseIt();
                }
                if (obj2 == keywordIif)
                {
                    return this.ParseIif();
                }
                if (obj2 == keywordNew)
                {
                    return this.ParseNew();
                }
                this.NextToken();
                return (Expression)obj2;
            }
            if (this.symbols.TryGetValue(this.token.text, out obj2) || ((this.externals != null) && this.externals.TryGetValue(this.token.text, out obj2)))
            {
                Expression expression = obj2 as Expression;
                if (expression == null)
                {
                    expression = Expression.Constant(obj2);
                }
                else
                {
                    LambdaExpression lambda = expression as LambdaExpression;
                    if (lambda != null)
                    {
                        return this.ParseLambdaInvocation(lambda);
                    }
                }
                this.NextToken();
                return expression;
            }
            if (this.it == null)
            {
                throw this.ParseError("UnknownIdentifier {0}", new object[] { this.token.text });
            }
            return this.ParseMemberAccess(null, this.it);
        }

        private Expression ParseIif()
        {
            int pos = this.token.pos;
            this.NextToken();
            Expression[] expressionArray = this.ParseArgumentList();
            if (expressionArray.Length != 3)
            {
                throw this.ParseError(pos, "IifRequiresThreeArgs", new object[0]);
            }
            return this.GenerateConditional(expressionArray[0], expressionArray[1], expressionArray[2], pos);
        }

        private Expression ParseIntegerLiteral()
        {
            long num2;
            this.ValidateToken(TokenId.IntegerLiteral);
            string text = this.token.text;
            if (text[0] != '-')
            {
                ulong num;
                if (!ulong.TryParse(text, out num))
                {
                    throw this.ParseError("InvalidIntegerLiteral {0}", new object[] { text });
                }
                this.NextToken();
                if (num <= 2147483647L)
                {
                    return this.CreateLiteral((int)num, text);
                }
                if (num <= 4294967295L)
                {
                    return this.CreateLiteral((uint)num, text);
                }
                if (num <= 9223372036854775807L)
                {
                    return this.CreateLiteral((long)num, text);
                }
                return this.CreateLiteral(num, text);
            }
            if (!long.TryParse(text, out num2))
            {
                throw this.ParseError("InvalidIntegerLiteral {0}", new object[] { text });
            }
            this.NextToken();
            if ((num2 >= -2147483648L) && (num2 <= 2147483647L))
            {
                return this.CreateLiteral((int)num2, text);
            }
            return this.CreateLiteral(num2, text);
        }

        private Expression ParseIt()
        {
            if (this.it == null)
            {
                throw this.ParseError("NoItInScope", new object[0]);
            }
            this.NextToken();
            return this.it;
        }

        private Expression ParseLambdaInvocation(LambdaExpression lambda)
        {
            MethodBase base2;
            int pos = this.token.pos;
            this.NextToken();
            Expression[] args = this.ParseArgumentList();
            if (this.FindMethod(lambda.Type, "Invoke", false, args, out base2) != 1)
            {
                throw this.ParseError(pos, "ArgsIncompatibleWithLambda", new object[0]);
            }
            return Expression.Invoke(lambda, args);
        }

        private Expression ParseLogicalAnd()
        {
            Expression left = this.ParseComparison();
            while ((this.token.id == TokenId.DoubleAmphersand) || this.TokenIdentifierIs("and"))
            {
                Token token = this.token;
                this.NextToken();
                Expression right = this.ParseComparison();
                this.CheckAndPromoteOperands(typeof(ILogicalSignatures), token.text, ref left, ref right, token.pos);
                left = Expression.AndAlso(left, right);
            }
            return left;
        }

        private Expression ParseLogicalOr()
        {
            Expression left = this.ParseLogicalAnd();
            while ((this.token.id == TokenId.DoubleBar) || this.TokenIdentifierIs("or"))
            {
                Token token = this.token;
                this.NextToken();
                Expression right = this.ParseLogicalAnd();
                this.CheckAndPromoteOperands(typeof(ILogicalSignatures), token.text, ref left, ref right, token.pos);
                left = Expression.OrElse(left, right);
            }
            return left;
        }

        private Expression ParseMemberAccess(Type type, Expression instance)
        {
            MethodBase base2;
            if (instance != null)
            {
                type = instance.Type;
            }
            int pos = this.token.pos;
            string identifier = this.GetIdentifier();
            this.NextToken();
            if (this.token.id != TokenId.OpenParen)
            {
                MemberInfo info2 = this.FindPropertyOrField(type, identifier, instance == null);
                if (info2 == null)
                {
                    throw this.ParseError(pos, "UnknownPropertyOrField {0} {1}", new object[] { identifier, GetTypeName(type) });
                }
                if (info2 is PropertyInfo)
                {
                    return Expression.Property(instance, (PropertyInfo)info2);
                }
                return Expression.Field(instance, (FieldInfo)info2);
            }
            if ((instance != null) && (type != typeof(string)))
            {
                Type type2 = FindGenericType(typeof(IEnumerable<>), type);
                if (type2 != null)
                {
                    Type elementType = type2.GetGenericArguments()[0];
                    return this.ParseAggregate(instance, elementType, identifier, pos);
                }
            }
            Expression[] args = this.ParseArgumentList();
            switch (this.FindMethod(type, identifier, instance == null, args, out base2))
            {
                case 0:
                    throw this.ParseError(pos, "NoApplicableMethod {0} {1}", new object[] { identifier, GetTypeName(type) });

                case 1:
                    {
                        MethodInfo method = (MethodInfo)base2;
                        if (!IsPredefinedType(method.DeclaringType))
                        {
                            throw this.ParseError(pos, "MethodsAreInaccessible {0}", new object[] { GetTypeName(method.DeclaringType) });
                        }
                        if (method.ReturnType == typeof(void))
                        {
                            throw this.ParseError(pos, "MethodIsVoid {0} {1}", new object[] { identifier, GetTypeName(method.DeclaringType) });
                        }
                        return Expression.Call(instance, method, args);
                    }
            }
            throw this.ParseError(pos, "AmbiguousMethodInvocation {0} {1}", new object[] { identifier, GetTypeName(type) });
        }

        private Expression ParseMultiplicative()
        {
            Expression left = this.ParseUnary();
            while (((this.token.id == TokenId.Asterisk) || (this.token.id == TokenId.Slash)) || ((this.token.id == TokenId.Percent) || this.TokenIdentifierIs("mod")))
            {
                Token token = this.token;
                this.NextToken();
                Expression right = this.ParseUnary();
                this.CheckAndPromoteOperands(typeof(IArithmeticSignatures), token.text, ref left, ref right, token.pos);
                switch (token.id)
                {
                    case TokenId.Identifier:
                    case TokenId.Percent:
                        goto Label_0075;

                    case TokenId.Asterisk:
                        left = Expression.Multiply(left, right);
                        break;

                    case TokenId.Slash:
                        {
                            left = Expression.Divide(left, right);
                            continue;
                        }
                }
                continue;
            Label_0075:
                left = Expression.Modulo(left, right);
            }
            return left;
        }

        private Expression ParseNew()
        {
            int num;
            string identifier;
            this.NextToken();
            this.ValidateToken(TokenId.OpenParen, "OpenParenExpected");
            this.NextToken();
            List<DynamicProperty> properties = new List<DynamicProperty>();
            List<Expression> list2 = new List<Expression>();
        Label_0025:
            num = this.token.pos;
            Expression item = this.ParseExpression();
            if (this.TokenIdentifierIs("as"))
            {
                this.NextToken();
                identifier = this.GetIdentifier();
                this.NextToken();
            }
            else
            {
                MemberExpression expression2 = item as MemberExpression;
                if (expression2 == null)
                {
                    throw this.ParseError(num, "MissingAsClause", new object[0]);
                }
                identifier = expression2.Member.Name;
            }
            list2.Add(item);
            properties.Add(new DynamicProperty(identifier, item.Type));
            if (this.token.id == TokenId.Comma)
            {
                this.NextToken();
                goto Label_0025;
            }
            this.ValidateToken(TokenId.CloseParen, "CloseParenOrCommaExpected");
            this.NextToken();
            Type type = DynamicExpression.CreateClass(properties);
            MemberBinding[] bindings = new MemberBinding[properties.Count];
            for (int i = 0; i < bindings.Length; i++)
            {
                bindings[i] = Expression.Bind(type.GetProperty(properties[i].Name), list2[i]);
            }
            return Expression.MemberInit(Expression.New(type), bindings);
        }

        private static object ParseNumber(string text, Type type)
        {
            switch (Type.GetTypeCode(GetNonNullableType(type)))
            {
                case TypeCode.SByte:
                    sbyte num;
                    if (!sbyte.TryParse(text, out num))
                    {
                        break;
                    }
                    return num;

                case TypeCode.Byte:
                    byte num2;
                    if (!byte.TryParse(text, out num2))
                    {
                        break;
                    }
                    return num2;

                case TypeCode.Int16:
                    short num3;
                    if (!short.TryParse(text, out num3))
                    {
                        break;
                    }
                    return num3;

                case TypeCode.UInt16:
                    ushort num4;
                    if (!ushort.TryParse(text, out num4))
                    {
                        break;
                    }
                    return num4;

                case TypeCode.Int32:
                    int num5;
                    if (!int.TryParse(text, out num5))
                    {
                        break;
                    }
                    return num5;

                case TypeCode.UInt32:
                    uint num6;
                    if (!uint.TryParse(text, out num6))
                    {
                        break;
                    }
                    return num6;

                case TypeCode.Int64:
                    long num7;
                    if (!long.TryParse(text, out num7))
                    {
                        break;
                    }
                    return num7;

                case TypeCode.UInt64:
                    ulong num8;
                    if (!ulong.TryParse(text, out num8))
                    {
                        break;
                    }
                    return num8;

                case TypeCode.Single:
                    float num9;
                    if (!float.TryParse(text, out num9))
                    {
                        break;
                    }
                    return num9;

                case TypeCode.Double:
                    double num10;
                    if (!double.TryParse(text, out num10))
                    {
                        break;
                    }
                    return num10;

                case TypeCode.Decimal:
                    decimal num11;
                    if (!decimal.TryParse(text, out num11))
                    {
                        break;
                    }
                    return num11;
            }
            return null;
        }

        public IEnumerable<DynamicOrdering> ParseOrdering()
        {
            Expression expression;
            List<DynamicOrdering> list = new List<DynamicOrdering>();
        Label_0006:
            expression = this.ParseExpression();
            bool flag = true;
            if (this.TokenIdentifierIs("asc") || this.TokenIdentifierIs("ascending"))
            {
                this.NextToken();
            }
            else if (this.TokenIdentifierIs("desc") || this.TokenIdentifierIs("descending"))
            {
                this.NextToken();
                flag = false;
            }
            list.Add(new DynamicOrdering { Selector = expression, Ascending = flag });
            if (this.token.id == TokenId.Comma)
            {
                this.NextToken();
                goto Label_0006;
            }
            this.ValidateToken(TokenId.End, "SyntaxError");
            return list;
        }

        private Expression ParseParenExpression()
        {
            this.ValidateToken(TokenId.OpenParen, "OpenParenExpected");
            this.NextToken();
            Expression expression = this.ParseExpression();
            this.ValidateToken(TokenId.CloseParen, "CloseParenOrOperatorExpected");
            this.NextToken();
            return expression;
        }

        private Expression ParsePrimary()
        {
            Expression instance = this.ParsePrimaryStart();
        Label_0007:
            while (this.token.id == TokenId.Dot)
            {
                this.NextToken();
                instance = this.ParseMemberAccess(null, instance);
            }
            if (this.token.id == TokenId.OpenBracket)
            {
                instance = this.ParseElementAccess(instance);
                goto Label_0007;
            }
            return instance;
        }

        private Expression ParsePrimaryStart()
        {
            switch (this.token.id)
            {
                case TokenId.Identifier:
                    return this.ParseIdentifier();

                case TokenId.StringLiteral:
                    return this.ParseStringLiteral();

                case TokenId.IntegerLiteral:
                    return this.ParseIntegerLiteral();

                case TokenId.RealLiteral:
                    return this.ParseRealLiteral();

                case TokenId.OpenParen:
                    return this.ParseParenExpression();
            }
            throw this.ParseError("ExpressionExpected", new object[0]);
        }

        private Expression ParseRealLiteral()
        {
            this.ValidateToken(TokenId.RealLiteral);
            string text = this.token.text;
            object obj2 = null;
            switch (text[text.Length - 1])
            {
                case 'F':
                case 'f':
                    float num;
                    if (float.TryParse(text.Substring(0, text.Length - 1), out num))
                    {
                        obj2 = num;
                    }
                    break;

                default:
                    double num2;
                    if (double.TryParse(text, out num2))
                    {
                        obj2 = num2;
                    }
                    break;
            }
            if (obj2 == null)
            {
                throw this.ParseError("InvalidRealLiteral {0}", new object[] { text });
            }
            this.NextToken();
            return this.CreateLiteral(obj2, text);
        }

        private Expression ParseStringLiteral()
        {
            this.ValidateToken(TokenId.StringLiteral);
            char ch = this.token.text[0];
            string text = this.token.text.Substring(1, this.token.text.Length - 2);
            int startIndex = 0;
            while (true)
            {
                int index = text.IndexOf(ch, startIndex);
                if (index < 0)
                {
                    break;
                }
                text = text.Remove(index, 1);
                startIndex = index + 1;
            }
            if (ch == '\'')
            {
                if (text.Length != 1)
                {
                    throw this.ParseError("InvalidCharacterLiteral", new object[0]);
                }
                this.NextToken();
                return this.CreateLiteral(text[0], text);
            }
            this.NextToken();
            return this.CreateLiteral(text, text);
        }

        private Expression ParseTypeAccess(Type type)
        {
            MethodBase base2;
            int pos = this.token.pos;
            this.NextToken();
            if (this.token.id == TokenId.Question)
            {
                if (!type.IsValueType || IsNullableType(type))
                {
                    throw this.ParseError(pos, "TypeHasNoNullableForm {0}", new object[] { GetTypeName(type) });
                }
                type = typeof(Nullable<>).MakeGenericType(new Type[] { type });
                this.NextToken();
            }
            if (this.token.id != TokenId.OpenParen)
            {
                this.ValidateToken(TokenId.Dot, "DotOrOpenParenExpected");
                this.NextToken();
                return this.ParseMemberAccess(type, null);
            }
            Expression[] args = this.ParseArgumentList();
            switch (this.FindBestMethod(type.GetConstructors(), args, out base2))
            {
                case 0:
                    if (args.Length != 1)
                    {
                        throw this.ParseError(pos, "NoMatchingConstructor {0}", new object[] { GetTypeName(type) });
                    }
                    return this.GenerateConversion(args[0], type, pos);

                case 1:
                    return Expression.New((ConstructorInfo)base2, args);
            }
            throw this.ParseError(pos, "AmbiguousConstructorInvocation {0}", new object[] { GetTypeName(type) });
        }

        private Expression ParseUnary()
        {
            if (((this.token.id != TokenId.Minus) && (this.token.id != TokenId.Exclamation)) && !this.TokenIdentifierIs("not"))
            {
                return this.ParsePrimary();
            }
            Token token = this.token;
            this.NextToken();
            if ((token.id == TokenId.Minus) && ((this.token.id == TokenId.IntegerLiteral) || (this.token.id == TokenId.RealLiteral)))
            {
                this.token.text = "-" + this.token.text;
                this.token.pos = token.pos;
                return this.ParsePrimary();
            }
            Expression expr = this.ParseUnary();
            if (token.id == TokenId.Minus)
            {
                this.CheckAndPromoteOperand(typeof(INegationSignatures), token.text, ref expr, token.pos);
                return Expression.Negate(expr);
            }
            this.CheckAndPromoteOperand(typeof(INotSignatures), token.text, ref expr, token.pos);
            return Expression.Not(expr);
        }

        private void ProcessParameters(ParameterExpression[] parameters)
        {
            foreach (ParameterExpression expression in parameters)
            {
                if (!string.IsNullOrEmpty(expression.Name))
                {
                    this.AddSymbol(expression.Name, expression);
                }
            }
            if ((parameters.Length == 1) && string.IsNullOrEmpty(parameters[0].Name))
            {
                this.it = parameters[0];
            }
        }

        private void ProcessValues(object[] values)
        {
            for (int i = 0; i < values.Length; i++)
            {
                object obj2 = values[i];
                if ((i == (values.Length - 1)) && (obj2 is IDictionary<string, object>))
                {
                    this.externals = (IDictionary<string, object>)obj2;
                }
                else
                {
                    this.AddSymbol("@" + i.ToString(CultureInfo.InvariantCulture), obj2);
                }
            }
        }

        private Expression PromoteExpression(Expression expr, Type type, bool exact)
        {
            if (expr.Type == type)
            {
                return expr;
            }
            if (expr is ConstantExpression)
            {
                ConstantExpression key = (ConstantExpression)expr;
                if (key == nullLiteral)
                {
                    if (!type.IsValueType || IsNullableType(type))
                    {
                        return Expression.Constant(null, type);
                    }
                }
                else
                {
                    string str;
                    if (this.literals.TryGetValue(key, out str))
                    {
                        Type nonNullableType = GetNonNullableType(type);
                        object obj2 = null;
                        switch (Type.GetTypeCode(key.Type))
                        {
                            case TypeCode.Int32:
                            case TypeCode.UInt32:
                            case TypeCode.Int64:
                            case TypeCode.UInt64:
                                obj2 = ParseNumber(str, nonNullableType);
                                break;

                            case TypeCode.Double:
                                if (nonNullableType == typeof(decimal))
                                {
                                    obj2 = ParseNumber(str, nonNullableType);
                                }
                                break;

                            case TypeCode.String:
                                obj2 = ParseEnum(str, nonNullableType);
                                break;
                        }
                        if (obj2 != null)
                        {
                            return Expression.Constant(obj2, type);
                        }
                    }
                }
            }
            if (!IsCompatibleWith(expr.Type, type))
            {
                return null;
            }
            if (!type.IsValueType && !exact)
            {
                return expr;
            }
            return Expression.Convert(expr, type);
        }

        private static IEnumerable<Type> SelfAndBaseClasses(Type type)
        {
            return new __SelfAndBaseClasses_d__5(-2) { m___type = type };
        }

        private static IEnumerable<Type> SelfAndBaseTypes(Type type)
        {
            if (type.IsInterface)
            {
                List<Type> types = new List<Type>();
                AddInterface(types, type);
                return types;
            }
            return SelfAndBaseClasses(type);
        }

        private void SetTextPos(int pos)
        {
            this.textPos = pos;
            this.ch = (this.textPos < this.textLen) ? this.text[this.textPos] : '\0';
        }

        private bool TokenIdentifierIs(string id)
        {
            return ((this.token.id == TokenId.Identifier) && string.Equals(id, this.token.text, StringComparison.OrdinalIgnoreCase));
        }

        private void ValidateDigit()
        {
            if (!char.IsDigit(this.ch))
            {
                throw this.ParseError(this.textPos, "DigitExpected", new object[0]);
            }
        }

        private void ValidateToken(TokenId t)
        {
            if (this.token.id != t)
            {
                throw this.ParseError("SyntaxError", new object[0]);
            }
        }

        private void ValidateToken(TokenId t, string errorMessage)
        {
            if (this.token.id != t)
            {
                throw this.ParseError(errorMessage, new object[0]);
            }
        }

        [CompilerGenerated]
        private sealed class __SelfAndBaseClasses_d__5: IEnumerable<Type>, IEnumerable, IEnumerator<Type>, IEnumerator, IDisposable
        {
            private int m___state;
            private Type m___current;
            public Type m___type;
            private int l__initialThreadId;
            public Type type;

            [DebuggerHidden]
            public __SelfAndBaseClasses_d__5(int m___state)
            {
                this.m___state = m___state;
                this.l__initialThreadId = Thread.CurrentThread.ManagedThreadId;
            }

            bool IEnumerator.MoveNext()
            {
                switch (this.m___state)
                {
                    case 0:
                        this.m___state = -1;
                        break;

                    case 1:
                        this.m___state = -1;
                        this.type = this.type.BaseType;
                        break;

                    default:
                        goto Label_0055;
                }
                if (this.type != null)
                {
                    this.m___current = this.type;
                    this.m___state = 1;
                    return true;
                }
            Label_0055:
                return false;
            }

            [DebuggerHidden]
            IEnumerator<Type> IEnumerable<Type>.GetEnumerator()
            {
                ExpressionParser.__SelfAndBaseClasses_d__5 d__;
                if ((Thread.CurrentThread.ManagedThreadId == this.l__initialThreadId) && (this.m___state == -2))
                {
                    this.m___state = 0;
                    d__ = this;
                }
                else
                {
                    d__ = new ExpressionParser.__SelfAndBaseClasses_d__5(0);
                }
                d__.type = this.m___type;
                return d__;
            }

            [DebuggerHidden]
            IEnumerator IEnumerable.GetEnumerator()
            {
                return this.AsEnumerable().GetEnumerator();
            }

            [DebuggerHidden]
            void IEnumerator.Reset()
            {
                throw new NotSupportedException();
            }

            void IDisposable.Dispose()
            {
            }

            Type IEnumerator<Type>.Current
            {
                [DebuggerHidden]
                get
                {
                    return this.m___current;
                }
            }

            object IEnumerator.Current
            {
                [DebuggerHidden]
                get
                {
                    return this.m___current;
                }
            }
        }

        private interface IAddSignatures: ExpressionParser.IArithmeticSignatures
        {
            void F(DateTime x, TimeSpan y);

            void F(DateTimeOffset x, TimeSpan y);

            void F(DateTime? x, TimeSpan? y);

            void F(DateTimeOffset? x, TimeSpan? y);

            void F(TimeSpan? x, TimeSpan? y);

            void F(TimeSpan x, TimeSpan y);
        }

        private interface IArithmeticSignatures
        {
            void F(decimal x, decimal y);

            void F(double x, double y);

            void F(int x, int y);

            void F(decimal? x, decimal? y);

            void F(double? x, double? y);

            void F(long x, long y);

            void F(int? x, int? y);

            void F(long? x, long? y);

            void F(float? x, float? y);

            void F(uint? x, uint? y);

            void F(ulong? x, ulong? y);

            void F(float x, float y);

            void F(uint x, uint y);

            void F(ulong x, ulong y);
        }

        private interface IEnumerableSignatures
        {
            void All(bool predicate);

            void Any();

            void Any(bool predicate);

            void Average(decimal? selector);

            void Average(decimal selector);

            void Average(double selector);

            void Average(long? selector);

            void Average(int selector);

            void Average(float? selector);

            void Average(long selector);

            void Average(double? selector);

            void Average(int? selector);

            void Average(float selector);

            void Count();

            void Count(bool predicate);

            void Max(object selector);

            void Min(object selector);

            void Sum(decimal? selector);

            void Sum(decimal selector);

            void Sum(double? selector);

            void Sum(int? selector);

            void Sum(long? selector);

            void Sum(double selector);

            void Sum(int selector);

            void Sum(long selector);

            void Sum(float? selector);

            void Sum(float selector);

            void Where(bool predicate);
        }

        private interface IEqualitySignatures: ExpressionParser.IRelationalSignatures, ExpressionParser.IArithmeticSignatures
        {
            void F(bool x, bool y);

            void F(bool? x, bool? y);

            void F(Guid x, Guid y);

            void F(Guid? x, Guid? y);
        }

        private interface ILogicalSignatures
        {
            void F(bool x, bool y);

            void F(bool? x, bool? y);
        }

        private interface INegationSignatures
        {
            void F(decimal x);

            void F(double x);

            void F(int x);

            void F(decimal? x);

            void F(long x);

            void F(double? x);

            void F(int? x);

            void F(long? x);

            void F(float? x);

            void F(float x);
        }

        private interface INotSignatures
        {
            void F(bool x);

            void F(bool? x);
        }

        private interface IRelationalSignatures: ExpressionParser.IArithmeticSignatures
        {
            void F(DateTime? x, DateTime? y);

            void F(char x, char y);

            void F(DateTime x, DateTime y);

            void F(DateTimeOffset x, DateTimeOffset y);

            void F(char? x, char? y);

            void F(DateTimeOffset? x, DateTimeOffset? y);

            void F(TimeSpan? x, TimeSpan? y);

            void F(string x, string y);

            void F(TimeSpan x, TimeSpan y);
        }

        private interface ISubtractSignatures: ExpressionParser.IAddSignatures, ExpressionParser.IArithmeticSignatures
        {
            void F(DateTime x, DateTime y);

            void F(DateTimeOffset x, DateTimeOffset y);

            void F(DateTime? x, DateTime? y);

            void F(DateTimeOffset? x, DateTimeOffset? y);
        }

        private class MethodData
        {
            public Expression[] Args;
            public MethodBase MethodBase;
            public ParameterInfo[] Parameters;
        }

        [StructLayout(LayoutKind.Sequential)]
        private struct Token
        {
            public ExpressionParser.TokenId id;
            public string text;
            public int pos;
        }

        private enum TokenId
        {
            Unknown,
            End,
            Identifier,
            StringLiteral,
            IntegerLiteral,
            RealLiteral,
            Exclamation,
            Percent,
            Amphersand,
            OpenParen,
            CloseParen,
            Asterisk,
            Plus,
            Comma,
            Minus,
            Dot,
            Slash,
            Colon,
            LessThan,
            Equal,
            GreaterThan,
            Question,
            OpenBracket,
            CloseBracket,
            Bar,
            ExclamationEqual,
            DoubleAmphersand,
            LessThanEqual,
            LessGreater,
            DoubleEqual,
            GreaterThanEqual,
            DoubleBar
        }
    }
}